/// <reference types="node"/>
/// <reference types="webdriver"/>

declare namespace WebDriver {
    interface ClientOptions {
        isDevTools: boolean;
        getPuppeteer: (...args: any[]) => any;
    }
}

declare namespace DevTools {
    function newSession(
        options?: WebDriver.Options,
        modifier?: (...args: any[]) => any,
        proto?: object,
        commandWrapper?: (commandName: string, fn: (...args: any[]) => any) => any
    ): Promise<Client>;

    function reloadSession(
        instance: Client
    ): Promise<Client>;

    // generated typings
        // devtools types
    interface Client {
        newSession(capabilities: object): object;
        deleteSession(): void;
        status(): object;
        getTimeouts(): object;
        setTimeouts(implicit?: number, pageLoad?: number, script?: number): void;
        getUrl(): string;
        navigateTo(url: string): string;
        back(): void;
        forward(): void;
        refresh(): void;
        getTitle(): string;
        getWindowHandle(): string;
        closeWindow(): void;
        switchToWindow(handle: string): void;
        createWindow(type: string): object;
        getWindowHandles(): string[];
        switchToFrame(id: (number|string|object|null)): void;
        switchToParentFrame(): void;
        getWindowRect(): object;
        setWindowRect(x: (number|null), y: (number|null), width: (number|null), height: (number|null)): object;
        maximizeWindow(): object;
        minimizeWindow(): object;
        fullscreenWindow(): object;
        findElement(using: string, value: string): string;
        findElements(using: string, value: string): string[];
        findElementFromElement(using: string, value: string): string;
        findElementsFromElement(using: string, value: string): string[];
        getActiveElement(): string;
        isElementSelected(): boolean;
        isElementDisplayed(): boolean;
        getElementAttribute(name: string): string;
        getElementProperty(name: string): string;
        getElementCSSValue(propertyName: string): string;
        getElementText(): string;
        getElementTagName(): string;
        getElementRect(): object;
        isElementEnabled(): boolean;
        elementClick(): void;
        elementClear(): void;
        elementSendKeys(text: string, value?: string[]): void;
        getPageSource(): string;
        executeScript(script: string, args?: (string|object|number|boolean|undefined)[]): any;
        executeAsyncScript(script: string, args: (string|object|number|boolean|undefined)[]): any;
        getAllCookies(): object[];
        addCookie(cookie: object): void;
        deleteAllCookies(): void;
        getNamedCookie(name: string): object;
        deleteCookie(name: string): void;
        performActions(actions: object[]): void;
        releaseActions(): void;
        dismissAlert(): void;
        acceptAlert(): void;
        getAlertText(): string;
        sendAlertText(text: string): void;
        takeScreenshot(): string;
        takeElementScreenshot(scroll?: boolean): string;
    }

}

declare module "devtools" {
    export default DevTools;
}
